@@ -1,13 +1,13 @@
 /*
- * Copyright (c) 1991-2003 Kyoto University
- * Copyright (c) 2000-2003 NAIST
+ * Copyright (c) 1991-2004 Kyoto University
+ * Copyright (c) 2000-2004 NAIST
  * All rights reserved
  */
 
 /* beam.c --- frame-synchronous beam search on 1st pass */
 /* use (1-best|word-pair) with static tree structure */
 
-/* $Id: beam.c,v 1.13 2003/12/22 09:39:23 ri Exp $ */
+/* $Id: beam.c,v 1.16 2004/04/27 04:55:31 ri Exp $ */
 
 #include <julius.h>
 
@@ -112,7 +112,7 @@ trace_backptr(WORD_ID wordseq_rt[MAXSEQNUM], /* word sequence (return) */
   int wordlen = 0;		/* word length of best sentence hypothesis */
   TRELLIS_ATOM *tretmp;
   LOGPROB langscore = 0.0;
-  static wordseq[MAXSEQNUM];	/* temporal: in reverse order */
+  static WORD_ID wordseq[MAXSEQNUM];	/* temporal: in reverse order */
   int i;
   
   /* initialize */
@@ -172,8 +172,10 @@ print_1pass_result(
   TRELLIS_ATOM *best;
   int last_time;
   LOGPROB total_lscore;
+#if defined(USE_DFA) || defined(SP_BREAK_CURRENT_FRAME)
   LOGPROB maxscore;
   TRELLIS_ATOM *tmp;
+#endif
 
   /* look for the first trellis word */
   for (last_time = framelen - 1; last_time >= 0; last_time--) {
@@ -329,7 +331,7 @@ bt_current_max_word(BACKTRELLIS *bt, int t, WORD_INFO *winfo)
 /*                functions to handle hypothesis tokens                  */
 /* -------------------------------------------------------------------- */
 
-/* Token structure:
+/* Token structure: */
    
 /* How tokens are managed:
    o  tlist[][] is a token stocker.  It holds all tokens in sequencial
@@ -352,7 +354,9 @@ static int totalnodenum;	/* allocated node num of above */
 static int tl,tn;		/* node id flip/flop */
 static int n_start, n_end;	/* (for heap sort) */
 
-static TRELLIS_ATOM bos;
+static TRELLIS_ATOM bos;	/* initial beginning of sentence token */
+
+static nodes_malloced = FALSE;	/* flag to check invalid free_node() */
 
 /* malloc work area */
 static void
@@ -367,6 +371,7 @@ malloc_nodes(int n)
   tindex[1]     = mymalloc(sizeof(TOKENID)*maxtnum);
   tnum[0] = tnum[1] = 0;
   expand_step = n;
+  nodes_malloced = TRUE;
 }
 /* expand work area */
 static void
@@ -383,11 +388,14 @@ expand_tlist()
 static void
 free_nodes()
 {
-  free(token);
-  free(tlist[0]);
-  free(tlist[1]);
-  free(tindex[0]);
-  free(tindex[1]);
+  if (nodes_malloced) {
+    free(token);
+    free(tlist[0]);
+    free(tlist[1]);
+    free(tindex[0]);
+    free(tindex[1]);
+    nodes_malloced = FALSE;
+  }
 }
 
 /* initialize token buffer */
@@ -685,7 +693,6 @@ detect_end_of_segment(BACKTRELLIS *backtrellis, int time, WCHMM_INFO *wchmm)
   LOGPROB maxscore = LOG_ZERO;
   TRELLIS_ATOM *tremax = NULL;
   int count = 0;
-  char *p;
   boolean detected = FALSE;
 
   /* look for the best trellis word on the given time frame */
@@ -809,7 +816,9 @@ init_nodescore(HTK_Param *param, WCHMM_INFO *wchmm)
 {
   TOKENID newid;
   TOKEN2 *new;
+#ifdef USE_NGRAM
   WORD_ID beginword;
+#endif
   int node;
   int i, j;
 
@@ -1011,18 +1020,21 @@ get_back_trellis_init(HTK_Param *param,	/* parameter vectors of speech data (onl
 
 /* local work area for get_back_trellis_proceed() */
 static LOGPROB maxscore, minscore;
-static LOGPROB tmpprob;
 static TRELLIS_ATOM *tre;
-static int node, stid, isoid;
+static int node, stid;
 static A_CELL *ac;
-static LOGPROB *iwparray;
 static int next_node2;
+#ifdef USE_NGRAM
+static LOGPROB tmpprob;
+static LOGPROB *iwparray;
+#endif
 #ifdef UNIGRAM_FACTORING
 /* for wordend processing with 1-gram factoring */
 static LOGPROB wordend_best_score; /* best score of wordend nodes */
 static int wordend_best_node;	/* node id of the best wordend nodes */
 static TRELLIS_ATOM *wordend_best_tre; /* trellis word corresponds to above */
 static WORD_ID wordend_best_last_cword;	/* last_cword of above */
+static int isoid;
 #endif
 
 /*****************************************************/
@@ -1631,17 +1643,34 @@ LOGPROB
 finalize_1st_pass(BACKTRELLIS *backtrellis, WORD_INFO *winfo, int len)
 {
   LOGPROB lastscore;
-  
+  int mseclen;
+  boolean ok_p;
+ 
+  backtrellis->framelen = len;
+
   /* $BC18l%H%l%j%9(B(backtrellis) $B$r@0M}(B: $B%H%l%j%9C18l$N:FG[CV$H%=!<%H(B */
   /* re-arrange backtrellis: index them by frame, and sort by word ID */
-  backtrellis->framelen = len;
   bt_relocate_rw(backtrellis);
   bt_sort_rw(backtrellis);
+
+  /* rejectshort $B;XDj;~(B, $BF~NO$,C;$1$l$P$3$3$GBh(B1$B%Q%97k2L$r=PNO$7$J$$(B */
+  /* $B$3$3$G$O(B reject $B$O$^$@9T$o$:(B, $B8e$G(B reject $B$9$k(B */
+  /* suppress 1st pass output if -rejectshort and input shorter than specified */
+  /* the actual rejection will be done later at main */
+  ok_p = TRUE;
+  if (rejectshortlen > 0) {
+    mseclen = (float)len * (float)smpPeriod * (float)fshift / 10000.0;
+    if (mseclen < rejectshortlen) {
+      ok_p = FALSE;
+    }
+  }
   
   /* $B7k2L$rI=<($9$k(B (best $B2>@b(B)*/
   /* output 1st pass result (best hypothesis) */
   if (verbose_flag && (!progout_flag)) j_printerr("\n");
-  lastscore = print_1pass_result(backtrellis, len, winfo);
+  if (ok_p) {
+    lastscore = print_1pass_result(backtrellis, len, winfo);
+  }
 
 #ifdef USE_NGRAM
   /* free succesor cache */
@@ -1664,7 +1693,7 @@ finalize_1st_pass(BACKTRELLIS *backtrellis, WORD_INFO *winfo, int len)
 void
 finalize_segment(BACKTRELLIS *backtrellis, HTK_Param *param, int len)
 {
-  int t, v;
+  int t;
 
   /* $B%H%l%j%9;O=*C<$K$*$1$k:GL`C18l$rBh(B2$B%Q%9$N;O=*C<C18l$H$7$F3JG<(B */
   /* fix initial/last word hypothesis of the next 2nd pass to the best
