@@ -1,12 +1,12 @@
 /*
- * Copyright (c) 1991-2003 Kyoto University
- * Copyright (c) 2000-2003 NAIST
+ * Copyright (c) 1991-2004 Kyoto University
+ * Copyright (c) 2000-2004 NAIST
  * All rights reserved
  */
 
 /* m_options.c --- process options and set values */
 
-/* $Id: m_options.c,v 1.21 2004/01/08 08:42:17 ri Exp $ */
+/* $Id: m_options.c,v 1.23 2004/04/27 04:55:31 ri Exp $ */
 
 #include <julius.h>
 
@@ -92,6 +92,9 @@ opt_parse(int argc, char *argv[], char *cwd)
     } else if (strmatch(argv[i],"-record")) {	/* record speech data*/
       record_dirname = NEXTARG;
       continue;
+    } else if (strmatch(argv[i],"-rejectshort")) { /* short input rejection */
+      rejectshortlen = atoi(NEXTARG);
+      continue;
     } else if (strmatch(argv[i],"-module")) { /* enable module mode */
       module_mode = TRUE;
       if (i+1 < argc) {
