@@ -1,13 +1,13 @@
 /*
- * Copyright (c) 1991-2003 Kyoto University
- * Copyright (c) 2000-2003 NAIST
+ * Copyright (c) 1991-2004 Kyoto University
+ * Copyright (c) 2000-2004 NAIST
  * All rights reserved
  */
 
 /* search_bestfirst_v1.c --- hypothesis handling and scoring functions
    for backscan algorithm */
 
-/* $Id: search_bestfirst_v1.c,v 1.11 2003/12/11 07:32:43 ri Exp $ */
+/* $Id: search_bestfirst_v1.c,v 1.13 2004/03/23 03:00:16 ri Exp $ */
 
 #include <julius.h>
 #ifndef PASS2_STRICT_IWCD
@@ -718,8 +718,11 @@ next_word(NODE *now, NODE *new,	NEXTWORD *nword, HTK_Param *param, BACKTRELLIS *
   int   i;
   int   startt;
   int word;
-  LOGPROB totalscore, tmpp;
+  LOGPROB tmpp;
+#ifndef WORD_GRAPH
+  LOGPROB totalscore;
   TRELLIS_ATOM *tre;
+#endif
 
   new->score = LOG_ZERO;
 
@@ -947,9 +950,11 @@ start_word(
 {
   HMM_Logical *newphone;
   WORD_ID word;
-  TRELLIS_ATOM *tre = NULL;
   LOGPROB tmpp;
+#ifndef WORD_GRAPH
   int t;
+  TRELLIS_ATOM *tre = NULL;
+#endif
 
   /* initialize data */
   word = nword->id;
