@@ -1,12 +1,12 @@
 /*
- * Copyright (c) 1991-2003 Kyoto University
- * Copyright (c) 2000-2003 NAIST
+ * Copyright (c) 1991-2004 Kyoto University
+ * Copyright (c) 2000-2004 NAIST
  * All rights reserved
  */
 
 /* outprob.c --- top routine of calculate output probability  */
 
-/* $Id: outprob.c,v 1.6 2003/09/29 06:01:23 ri Exp $ */
+/* $Id: outprob.c,v 1.9 2004/03/23 03:00:16 ri Exp $ */
 
 #include <sent/stddefs.h>
 #include <sent/speech.h>
@@ -41,7 +41,6 @@ outprob_cache_init()
 boolean
 outprob_cache_prepare()
 {
-  int i, size;
   int s,t;
 
   /* clear already allocated area */
@@ -119,6 +118,51 @@ outprob_state(
   return(outp);
 }
 
+/* return average of N-beat outprob for cd state set */
+static LOGPROB *maxprobs;
+static int maxn;
+void
+outprob_cd_nbest_init(int num)
+{
+  maxprobs = (LOGPROB *)mymalloc(sizeof(LOGPROB) * num);
+  maxn = num;
+}
+  
+static LOGPROB
+outprob_cd_nbest(int t, CD_State_Set *lset, HTK_Param *param)
+{
+  LOGPROB prob;
+  int i, k, n;
+
+  n = 0;
+  for(i=0;i<lset->num;i++) {
+    prob = outprob_state(t, lset->s[i], param);
+    /*printf("\t\t%d:%f\n", i, prob);*/
+    if (prob <= LOG_ZERO) continue;
+    if (n == 0 || prob <= maxprobs[n-1]) {
+      if (n == maxn) continue;
+      maxprobs[n] = prob;
+      n++;
+    } else {
+      for(k=0; k<n; k++) {
+	if (prob > maxprobs[k]) {
+	  memmove(&(maxprobs[k+1]), &(maxprobs[k]),
+		  sizeof(LOGPROB) * (n - k - ( (n == maxn) ? 1 : 0)));
+	  maxprobs[k] = prob;
+	  break;
+	}
+      }
+      if (n < maxn) n++;
+    }
+  }
+  prob = 0.0;
+  for(i=0;i<n;i++) {
+    /*printf("\t\t\t- %d: %f\n", i, maxprobs[i]);*/
+    prob += maxprobs[i];
+  }
+  return(prob/(float)n);
+}
+  
 /* return maximum outprob for cd state set */
 static LOGPROB
 outprob_cd_max(int t, CD_State_Set *lset, HTK_Param *param)
@@ -154,11 +198,19 @@ outprob_cd_avg(int t, CD_State_Set *lset, HTK_Param *param)
 LOGPROB
 outprob_cd(int t, CD_State_Set *lset, HTK_Param *param)
 {
-  if (!OP_hmminfo->prefer_cdset_avg) {
-    return(outprob_cd_max(t, lset, param));
-  } else {
-    return(outprob_cd_avg(t, lset, param));
+  LOGPROB ret;
+  switch(OP_hmminfo->cdset_method) {
+  case IWCD_AVG:
+    ret = outprob_cd_avg(t, lset, param);
+    break;
+  case IWCD_MAX:
+    ret = outprob_cd_max(t, lset, param);
+    break;
+  case IWCD_NBEST:
+    ret = outprob_cd_nbest(t, lset, param);
+    break;
   }
+  return(ret);
 }
   
 
